#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          02-07-2018
//      Created for Project :   FLAME
"""

from sqlalchemy import Column, String, Integer, UniqueConstraint, and_
from sqlalchemy.dialects.postgresql import JSONB
from clmcservice.models.meta import Base


class ServiceFunctionChain(Base):
    """
    This class defines the service function chain model of the config API, declaring the relations between individual service functions per service function chain.
    """

    __tablename__ = 'sfchain'  # table name in the PostgreSQL database

    __table_args__ = (UniqueConstraint('sfc', 'sfc_i'),)  # defines a unique constraint across 2 columns - sfc, sfc_i

    uid = Column(Integer, primary_key=True, autoincrement=True, nullable=False)  # a primary key integer field (auto incremented)

    sfc = Column(String, nullable=False)  # service function chain label
    sfc_i = Column(String, nullable=False)  # service function chain instance identifier
    chain = Column(JSONB, nullable=False)  # the service function chain graph represented by a python dictionary (JSON object essentially)

    @property
    def json(self):
        """
        Converts an instance of a ServiceFunctionChain to JSON format.

        :return: a python dictionary object
        """

        fields = {c.name: getattr(self, c.name) for c in self.__table__.columns}
        fields.pop("uid")

        return fields

    @staticmethod
    def required_columns():
        """
        Returns the required columns for constructing a valid instance.
        :return: a generator object
        """

        return tuple(column.name for column in ServiceFunctionChain.__table__.columns if column.name != "uid")

    @staticmethod
    def constrained_columns():
        """
        :return: the columns that are uniquely identifying an instance of this model.
        """

        return tuple(column.name for column in ServiceFunctionChain.__table_args__[0].columns)

    @staticmethod
    def get(sfc, sfc_i):
        """
        Gets the instance matching the unique constraint or None if not existing.

        :param sfc: service function chain id
        :param sfc_i: service function chain instance id

        :return: the first object from the result set that matches the unique constraint or None
        """

        return ServiceFunctionChain.query().filter(and_(ServiceFunctionChain.sfc == sfc, ServiceFunctionChain.sfc_i == sfc_i)).first()

    @staticmethod
    def exists(sfc, sfc_i):
        """
        Checks if an instance matching the unique constraint exists.

        :param sfc: service function chain id
        :param sfc_i: service function chain instance id

        :return: True if exists, False otherwise
        """

        return ServiceFunctionChain.get(sfc, sfc_i) is not None
